%% Extract photobleaching ROIs and fit traces in green channel (left)
% filename, double (motor_##_MMImages.ome.tif) or string
% thrs, double or 'auto' or 'slider', custom, automatic or slider-adjusted binarization threshold for im2bw() in centroids() in findspots()

% change prot_img, prot_spots and exclude_on_edges bounds

% 161003 Evolve, TetraSpeck calibration
% upper left pixel in green channel = (5,17)
% bottom right pixel in green channel = (228,505)
% window size = 224 px (width) by 489 px (height)

% 161129 Evolve Delta, TetraSpeck calibration
% upper left pixel in green channel = (10,2)
% bottom right pixel in green channel = (243,510)
% window size = 234 px (width) by 509 px (height)

% 161201 Evolve, TetraSpeck calibration <<< current
% upper left pixel in green channel = (10,5)
% bottom right pixel in green channel = (236,508)
% window size = 227 px (width) by 504 px (height)

function gbleach(filename,thrs)

    exclude_on_edges = 1; % boolean, rejects coordinates within 5 px of border
    box_size = 10; % even integer for correct box size
    min_spots = 4; % minimum number of spots for auto-thresholding

    % filename = '/Users/kf656/Desktop/HK97/TIRF/161005 10 nM BOBO DNA motor no binning multiplier 200/motor_1_MMImages.ome.tif';

    % check for file and, if necessary, expand name
    if exist(strcat('motor_',num2str(filename),'_MMImages.ome.tif')) == 2
        filename = strcat('motor_',num2str(filename),'_MMImages.ome.tif');
    elseif exist(filename,'file') == 0
        disp('File not found.');
        return;
    end

    t = strsplit(filename,'/');
    t = strsplit(t{end},'.ome.tif');
    t = strsplit(t{1},'_MMImages');
    rootname = t{1};
    intfile = strcat(rootname,'/',rootname,'_green_intensities.csv');
    
    warning('off','MATLAB:imagesci:tiffmexutils:libtiffWarning'); % disables tag warnings when assigning tiff object
    
    % read tiff stack using LibTIFF routines (alternative to imread)
    info = imfinfo(filename);
    nframes = numel(info);
    tiff_obj = Tiff(filename,'r');
    tiff_stack = zeros(512,512,nframes);
    for k = 1:nframes
          tiff_obj.setDirectory(k);
          tiff_stack(:,:,k) = tiff_obj.read();
    end
    tiff_obj.close();

    % sum first 900 frames of green channel
    prot_img = sum(tiff_stack(5:508,10:236,1:900),3);

    % find centroids in green channel, custom or semi-automatically generated binarization threshold
    [prot_coord, nspots] = findspots(prot_img,thrs,exclude_on_edges);  % 0.125 for motor_1

    % extract boxes around matching coordinates in green channel (protein) as 1D cell array of 3D arrays/movies (y,x,t)
    for k = 1:nspots
        prot_spots{k} = tiff_stack(4+((prot_coord(k,2)-box_size/2+1):(prot_coord(k,2)+(box_size/2))),9+((prot_coord(k,1)-box_size/2+1):(prot_coord(k,1)+(box_size/2))),:);
        intensities{k} = transpose(squeeze(sum(sum(prot_spots{k},1),2))/(box_size^2)); 
    end

    % export intensities (csv)

    % create sub-directory, if absent
    if exist(rootname) ~= 7
        mkdir(rootname);
    end

    % intensity csv header
    f = fopen(intfile,'w');
    fprintf(f,'%s %f\n',['Binarization threshold: ' num2str(thrs)]);
    fprintf(f,'\n');
    fclose(f);

    for k=1:nspots
        f = fopen(intfile,'a');
        fprintf(f,strcat(num2str(prot_coord(k,1)),',',num2str(prot_coord(k,2)),',')); % csv content, first two columns, DNA coord
        fclose(f);
        dlmwrite(intfile,intensities{k},'-append'); % csv content, remaining columns, intensities
    end

    for k = 1:nspots
        if max(intensities{k})<1000
            pif_custom(prot_coord(k,:),intensities{k},rootname);
        end
    end

end

% local function, mark spots on image
function dispspots(img,coord)
    figure('Colormap', gray);
    image(img,'CDataMapping','scaled');
    hold on;
    axis image;
    plot(coord(:,1),coord(:,2),'ro','MarkerSize',5);
    hold off;
end

% local function, wrapper for finding centroids, custom, automatic and/or slider-adjusted binarization threshold
function [coord,nspots] = findspots(img,thrs,exclude_on_edges)
    if isnumeric(thrs) & thrs > 0 & thrs < 1
        coord = centroids(img,thrs);
        if exclude_on_edges == 1
            coord = round(coord(coord(:,1)>5 & coord(:,1)<222 & coord(:,2)>5 & coord(:,2)<499,:));  % check calibration
        else
            coord = round(coord);
        end
        nspots = size(coord,1);
        dispspots(img,coord);
    elseif strcmp(thrs,'auto') == 1 || strcmp(thrs,'slider') == 1
        if strcmp(thrs,'slider') == 1
            slider_on = 1;
        end
        thrs = graythresh(img);
        if thrs == 0
            thrs = 0.5;
        end
        nspots = 0;
        while nspots < 5 || nspots > 16
            if nspots < 5
                thrs = thrs - rand()*0.1;
                if thrs < 0
                    thrs = abs(thrs);
                end
            else
                thrs = thrs + rand()*0.1;
                while thrs > 1
                    thrs = thrs - 0.1;
                end
            end
            coord = centroids(img,thrs);
            if exclude_on_edges == 1
                coord = round(coord(coord(:,1)>5 & coord(:,1)<222 & coord(:,2)>5 & coord(:,2)<499,:));  % check calibration
            else
                coord = round(coord);
            end
            reject_close();
            nspots = size(coord,1);
        end
        disp(['Binarization threshold: ' num2str(thrs)]);
        soundobj = load('train.mat');
        % sound(soundobj.y);
        dispspots(img,coord);
        if slider_on == 1
            fig_image = gcf;
            set(gca, 'Position', [0.25 0.20 0.50 0.75]);
            uicontrol('Parent',fig_image,'Style','slider','Units','normalized','Position',[0.36 0.03 0.20 0.05],...
                        'Min',0,'Max',1,'Value',thrs,'Callback',@redisp);
            uicontrol('Parent',fig_image,'Style','pushbutton','Units','normalized','Position',[0.58 0.036 0.06 0.05],...
                        'String','Done','Callback',@done_button);
            txt = uicontrol('Parent',fig_image,'Style','text','Units','normalized','Position',[0.35 0.08 0.25 0.05],...
                            'String',['Threshold: ',num2str(thrs)]);
            uiwait(); 			
        end
    else
        disp('Invalid threshold.');
        return;
    end

    function done_button(source,event)
        uiresume();
        disp('Extracting spots...');
        close(gcf);
    end

    % nested function
    function redisp(source,event)
        thrs = source.Value;
        coord = centroids(img,thrs);
        if exclude_on_edges == 1
            coord = round(coord(coord(:,1)>5 & coord(:,1)<222 & coord(:,2)>5 & coord(:,2)<499,:));  % check calibration
        else
            coord = round(coord);
        end
        reject_close();
        nspots = size(coord,1);
        txt.String = strcat('Threshold:',num2str(thrs));
        image(img,'CDataMapping','scaled');
        hold on;
        axis image;
        plot(coord(:,1),coord(:,2),'ro','MarkerSize',5);
        hold off;
    end

    % nested function, filter for brighter of overlapping ROIs
    function reject_close()
        distances = squareform(pdist(coord));
        [row,col] = find(distances<5);
        close_ind = [row(row>col),col(row>col)];
        remove_ind = zeros(size(close_ind,1),1);
        for k = 1:size(close_ind,1)
            int1 = img(coord(close_ind(k,1),2),coord(close_ind(k,1),1));
            int2 = img(coord(close_ind(k,2),2),coord(close_ind(k,2),1));
            if int1 > int2
                remove_ind(k) = close_ind(k,2);
            else
                remove_ind(k) = close_ind(k,1);
            end
        end
        coord(remove_ind,:)=[];
    end

end



% prot_img = tiff_stack(17:505,5:228,:);    
% figure(2);
% image(prot_img,'CDataMapping','scaled');    
% prot_spots = centroids(prot_img,0.04);
% hold on
% plot(prot_spots(:,1),prot_spots(:,2),'r*');
% hold off;

% test=ckfilter(intensities{4},[10 20],5,3);plot(test(1:1000));hold on;plot(diff(test(1:1000)));hold off
% test=ckfilter(intensities{4},[10 20],5,3);figure(1);plot(test(1:1000));hold on;plot(diff(test(1:1000)));hold off;figure(2);histogram(test(36:400),25)